<?php
namespace FiloBlu\ProductUrlTools\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Magento\UrlRewrite\Service\V1\Data\UrlRewrite;
use Magento\UrlRewrite\Model\UrlPersistInterface;
use Magento\CatalogUrlRewrite\Model\ProductUrlRewriteGenerator;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Store\Model\Store;
use Magento\Framework\App\State;

class RegenerateProductUrlCommand extends Command
{
    /**
     * @var ProductUrlRewriteGenerator
     */
    protected $productUrlRewriteGenerator;

    /**
     * @var UrlPersistInterface
     */
    protected $urlPersist;

    /**
     * @var ProductRepositoryInterface
     */
    protected $collection;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $state;
    protected $logger;

    public function __construct(
        State $state,
        Collection $collection,
        ProductUrlRewriteGenerator $productUrlRewriteGenerator,
        UrlPersistInterface $urlPersist,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Catalog\Model\Product\Attribute\Source\Status $productStatus,
        \Magento\Catalog\Model\Product\Visibility $productVisibility
    ) {
        $this->state = $state;
        $this->collection = $collection;
        $this->productUrlRewriteGenerator = $productUrlRewriteGenerator;
        $this->urlPersist = $urlPersist;
        $this->logger = $logger;
        $this->productVisibility = $productVisibility;
        $this->productStatus = $productStatus;
        parent::__construct();
    }

    protected function configure()
    {
        $this->setName('tools:regenerateurl')
            ->setDescription('Regenerate url for given products, recommended launch regenerate url key first')
            ->addArgument(
                'pids',
                InputArgument::IS_ARRAY,
                'Products to regenerate'
            )
            ->addOption(
                'store', 's',
                InputOption::VALUE_REQUIRED,
                'Use the specific Store View',
                Store::DEFAULT_STORE_ID
            )
            ;
        return parent::configure();
    }

    public function execute(InputInterface $inp, OutputInterface $out)
    {
//      if (!$this->state->getAreaCode()) {
//          $this->state->setAreaCode('adminhtml');
//       }

        $store_id = $inp->getOption('store');

        $this->collection->addStoreFilter($store_id)->setStoreId($store_id);

        $pids = $inp->getArgument('pids');

        if( !empty($pids) )
            $this->collection->addIdFilter($pids);
        else {
            $this->collection->addAttributeToFilter('status', ['in' => $this->productStatus->getVisibleStatusIds()]);
            $this->collection->setVisibility($this->productVisibility->getVisibleInSiteIds());
        }

        $this->collection->addAttributeToSelect(['url_path','url_key']);
        $list = $this->collection->load();


        foreach($list as $product)
        {
            if($store_id === Store::DEFAULT_STORE_ID)
                $product->setStoreId($store_id);

            $product->setStoreId($store_id);

            $this->urlPersist->deleteByData([
                UrlRewrite::ENTITY_ID => $product->getId(),
                UrlRewrite::ENTITY_TYPE => ProductUrlRewriteGenerator::ENTITY_TYPE,
                UrlRewrite::REDIRECT_TYPE => 0,
                UrlRewrite::STORE_ID => $product->getStoreId()

            ]);

            try {


                $urls = $this->productUrlRewriteGenerator->generate($product);

                $this->urlPersist->replace($urls);
                
                $this->logger->info('processed url for product with id : '. $product->getId());

            }
            catch(\Exception $e) {
                $out->writeln('<error>Duplicated url for '. $product->getId() .' '. $e->getMessage(). ' </error>');
            }

        }
    }
}
